from __future__ import print_function, division, absolute_import
import sys
import screed


def perror(*args, **kwargs):
    '''Print an error to stderr, and exit with non-zero code.'''
    print(file=sys.stderr, *args, **kwargs)
    exit(1)


def output_frag_fasta(read, frag, stream, width=80):
    print('>', read.name, '_', frag.lhs, '_', frag.rhs, sep='', file=stream)
    seq = read.sequence[frag.lhs:frag.rhs]
    for start in range(0, len(seq), width):
        print(seq[start:start+width], file=stream)


def output_bed(name, start, stop, label, stream):
    print(name, start, stop, label, sep='\t', file=stream)


def seqfile_iter_frags(seqfile, digestor, minlen, maxlen):
    for read in screed.open(seqfile, parse_description=True):
        seq = read.sequence
        for frag in digestor.iter_fragments(seq, minlen=minlen, maxlen=maxlen):
            yield read, frag



#!/usr/bin/env python
from setuptools import setup
import versioneer


desc = """
RADsim: simulate reduced-represention sequencing platforms based on restriction
digestion, including RADseq and GBS.
"""

setup_requires = [
    'pytest-runner',
]

with open('requirements.txt') as fh:
    install_requires = [req.strip() for req in fh]

test_requires = [
    'pep8',
    'pylint',
    'pytest',
]

command_classes = versioneer.get_cmdclass()

setup(
    name="radsim",
    packages=['radsim', ],
    version=versioneer.get_version(),
    entry_points={
        'console_scripts': [
            'radsim-hist = radsim.main:hist_main',
            'radsim-digest = radsim.main:digest_main',
            'radsim-rebed = radsim.main:rebed_main',
        ],
    },
    cmdclass=command_classes,
    install_requires=install_requires,
    tests_require=test_requires,
    setup_requires=setup_requires,
    description=desc,
    author="Kevin Murray",
    author_email="spam@kdmurray.id.au",
    url="https://github.com/kdmurray91/radhax",
    keywords=[
        "bioinformatics",
        "RADseq",
        "GBS",
        "Genotyping by Sequencing",
        "Next-gen Sequencing",
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "License :: OSI Approved :: GNU General Public License v3 or later "
        "(GPLv3+)",
    ],
)



from __future__ import print_function, division, absolute_import
from collections import Counter
from argparse import ArgumentParser, FileType
import sys

import numpy as np
import screed

from .digest import Digest
from .utils import (
    output_frag_fasta,
    output_bed,
    perror,
    seqfile_iter_frags,
)


def add_common_args(ap):
    ap.add_argument('--genome', '-i', required=True, type=str,
                    help='Genome sequence (in fasta format)')
    ap.add_argument('--enzyme', '-e', required=True,
                    help='Restriction enzyme name')
    ap.add_argument('--enzyme2', '-r', default=None,
                    help='Second restriction enzyme name (for ddRAD, etc)')


def add_frag_len_args(ap):
    ap.add_argument('--max', '-x', default=10000, type=int,
                    help='Maximum fragment size')
    ap.add_argument('--min', '-n', default=1, type=int,
                    help='Minimum fragment size')


def hist_main():
    ap = ArgumentParser(description="Create histogram of fragment sizes")
    add_common_args(ap)
    add_frag_len_args(ap)
    ap.add_argument('--output', '-o', type=FileType('w'), default=sys.stdout,
                    help='Output file (default stdout)')
    ap.add_argument('--bins', '-b', type=int, default=100,
                    help='Number of bins in histogram')
    args = ap.parse_args()
    digestor = Digest(args.enzyme, args.enzyme2)

    sizes = []
    frags = seqfile_iter_frags(args.genome, digestor, minlen=args.min,
                               maxlen=args.max)
    for _, frag in frags:
        sizes.append(frag.len)
    sizes = np.array(sizes)
    counts, edges = np.histogram(sizes)
    last = edges[0]
    for i, count in enumerate(counts):
        range_str = "{}-{}".format(last, edges[i+1]-1)
        print(i+1, range_str, count, sep='\t', file=args.output)
        last = edges[i]


def digest_main():
    ap = ArgumentParser(description="Performs in-silico digestion of a genome")
    ap.add_argument('--fasta', type=FileType('w'),
                    help="Fasta file to output fragment sequences")
    ap.add_argument('--bed', type=FileType('w'),
                    help="Bed file to output fragment sequences")
    add_common_args(ap)
    add_frag_len_args(ap)
    args = ap.parse_args()
    if not (args.fasta or args.bed):
        ap.error("One of --fasta FILE or --bed FILE is required")
    digestor = Digest(args.enzyme, args.enzyme2)

    frags = seqfile_iter_frags(args.genome, digestor, minlen=args.min,
                               maxlen=args.max)
    for read, frag in frags:
        if args.fasta:
            output_frag_fasta(read, frag, args.fasta)
        if args.bed:
            site_name = '{}[{}]_{}[{}]'.format(frag.lhs, frag.lhs_enzyme,
                                               frag.rhs, frag.rhs_enzyme)
            output_bed(read.name, frag.lhs, frag.rhs, site_name, args.bed)


def rebed_main():
    ap = ArgumentParser(description="Produces a BED file containing RE sites")
    ap.add_argument('--output', '-o', type=FileType('w'), default=sys.stdout,
                    help='Output file (default stdout)')
    add_common_args(ap)
    args = ap.parse_args()
    digest = Digest(args.enzyme, args.enzyme2)

    for read in screed.open(args.genome, parse_description=True):
        seq = read.sequence
        for cut, enz in digest.re_sites(seq):
            output_bed(read.name, cut, cut + enz.size, str(enz), args.output)
